@@ -3,7 +3,7 @@ package Catalyst::Engine::FastCGI;
 use strict;
 use base 'Catalyst::Engine::CGI';
 eval "use FCGI";
-die "Please install FCGI\n" if $@;
+die "Unable to load the FCGI module, you may need to install it:\n$@\n" if $@;
 
 =head1 NAME
 
@@ -97,7 +97,7 @@ sub run {
     my %env;
     my $error = \*STDERR; # send STDERR to the web server
        $error = \*STDOUT  # send STDERR to stdout (a logfile)
-	 if $options->{keep_stderr}; # (if asked to)
+         if $options->{keep_stderr}; # (if asked to)
     
     my $request =
       FCGI::Request( \*STDIN, \*STDOUT, $error, \%env, $sock,
@@ -159,6 +159,15 @@ sub write {
         $self->prepare_write($c);
         $self->{_prepared_write} = 1;
     }
+    
+    # XXX: We can't use Engine's write() method because syswrite
+    # appears to return bogus values instead of the number of bytes
+    # written: http://www.fastcgi.com/om_archive/mail-archive/0128.html
+    
+    # Prepend the headers if they have not yet been sent
+    if ( my $headers = delete $self->{_header_buf} ) {
+        $buffer = $headers . $buffer;
+    }
 
     # FastCGI does not stream data properly if using 'print $handle',
     # but a syswrite appears to work properly.
