@@ -6,6 +6,8 @@ use base 'Class::Accessor::Fast';
 use IO::Socket qw[AF_INET inet_aton];
 use Carp;
 use utf8;
+use URI::http;
+use URI::https;
 use URI::QueryParam;
 
 __PACKAGE__->mk_accessors(
@@ -401,6 +403,8 @@ Returns the protocol (HTTP/1.0 or HTTP/1.1) used for the current request.
 
 =head2 $req->query_parameters
 
+=head2 $req->query_params
+
 Returns a reference to a hash containing query string (GET) parameters. Values can
 be either a scalar or an arrayref containing scalars.
 
