@@ -1,93 +1,90 @@
 package Catalyst::Stats;
 
-use Moose;
+use strict;
+use warnings;
 use Time::HiRes qw/gettimeofday tv_interval/;
 use Text::SimpleTable ();
-use Catalyst::Utils;
 use Tree::Simple qw/use_weak_refs/;
 use Tree::Simple::Visitor::FindByUID;
 
-use namespace::clean -except => 'meta';
+sub new {
+    my $class = shift;
 
-has enable => (is => 'rw', required => 1, default => sub{ 1 });
-has tree => (
-             is => 'ro',
-             required => 1,
-             default => sub{ Tree::Simple->new({t => [gettimeofday]}) },
-             handles => [qw/ accept traverse /],
-            );
-has stack => (
-              is => 'ro',
-              required => 1,
-              lazy => 1,
-              default => sub { [ shift->tree ] }
-             );
+    my $root = Tree::Simple->new({t => [gettimeofday]});
+    bless { 
+    enabled => 1,
+    stack => [ $root ],
+    tree => $root,
+    }, ref $class || $class;
+}
+
+sub enable {
+    my ($self, $enable) = @_;
+
+    $self->{enabled} = $enable;
+}
 
 sub profile {
     my $self = shift;
 
-    return unless $self->enable;
+    return unless $self->{enabled};
 
     my %params;
     if (@_ <= 1) {
-        $params{comment} = shift || "";
+    $params{comment} = shift || "";
     }
     elsif (@_ % 2 != 0) {
-        die "profile() requires a single comment parameter or a list of name-value pairs; found "
-            . (scalar @_) . " values: " . join(", ", @_);
+    die "profile() requires a single comment parameter or a list of name-value pairs; found " 
+        . (scalar @_) . " values: " . join(", ", @_);
     }
     else {
-        (%params) = @_;
-        $params{comment} ||= "";
+    (%params) = @_;
+    $params{comment} ||= "";
     }
 
     my $parent;
     my $prev;
     my $t = [ gettimeofday ];
-    my $stack = $self->stack;
 
     if ($params{end}) {
-        # parent is on stack; search for matching block and splice out
-        for (my $i = $#{$stack}; $i > 0; $i--) {
-            if ($stack->[$i]->getNodeValue->{action} eq $params{end}) {
-                my ($node) = splice(@{$stack}, $i, 1);
-                # Adjust elapsed on partner node
-                my $v = $node->getNodeValue;
-                $v->{elapsed} =  tv_interval($v->{t}, $t);
-                return $node->getUID;
-            }
+    # parent is on stack; search for matching block and splice out
+    for (my $i = $#{$self->{stack}}; $i > 0; $i--) {
+        if ($self->{stack}->[$i]->getNodeValue->{action} eq $params{end}) {
+        my $node = $self->{stack}->[$i];
+        splice(@{$self->{stack}}, $i, 1);
+        # Adjust elapsed on partner node
+        my $v = $node->getNodeValue;
+        $v->{elapsed} =  tv_interval($v->{t}, $t);
+        return $node->getUID;
         }
+    }
     # if partner not found, fall through to treat as non-closing call
     }
     if ($params{parent}) {
-        # parent is explicitly defined
-        $prev = $parent = $self->_get_uid($params{parent});
+    # parent is explicitly defined
+    $prev = $parent = $self->_get_uid($params{parent});
     }
     if (!$parent) {
-        # Find previous node, which is either previous sibling or parent, for ref time.
-        $prev = $parent = $stack->[-1] or return undef;
-        my $n = $parent->getChildCount;
-        $prev = $parent->getChild($n - 1) if $n > 0;
+    # Find previous node, which is either previous sibling or parent, for ref time.
+    $prev = $parent = $self->{stack}->[-1] or return undef;
+    my $n = $parent->getChildCount;
+    $prev = $parent->getChild($n - 1) if $n > 0;
     }
 
     my $node = Tree::Simple->new({
-        action  => $params{begin} || "",
-        t => $t,
-        elapsed => tv_interval($prev->getNodeValue->{t}, $t),
-        comment => $params{comment},
+    action  => $params{begin} || "",
+    t => $t, 
+    elapsed => tv_interval($prev->getNodeValue->{t}, $t),
+    comment => $params{comment},
     });
     $node->setUID($params{uid}) if $params{uid};
 
     $parent->addChild($node);
-    push(@{$stack}, $node) if $params{begin};
+    push(@{$self->{stack}}, $node) if $params{begin};
 
     return $node->getUID;
 }
 
-sub created {
-    return @{ shift->{tree}->getNodeValue->{t} };
-}
-
 sub elapsed {
     return tv_interval(shift->{tree}->getNodeValue->{t});
 }
@@ -95,10 +92,14 @@ sub elapsed {
 sub report {
     my $self = shift;
 
-    my $column_width = Catalyst::Utils::term_width() - 9 - 13;
-    my $t = Text::SimpleTable->new( [ $column_width, 'Action' ], [ 9, 'Time' ] );
+# close any remaining open nodes
+    for (my $i = $#{$self->{stack}}; $i > 0; $i--) {
+    $self->profile(end => $self->{stack}->[$i]->getNodeValue->{action});
+    }
+
+    my $t = Text::SimpleTable->new( [ 62, 'Action' ], [ 9, 'Time' ] );
     my @results;
-    $self->traverse(
+    $self->{tree}->traverse(
                 sub {
                 my $action = shift;
                 my $stat   = $action->getNodeValue;
@@ -108,10 +109,8 @@ sub report {
                       $stat->{elapsed},
                       $stat->{action} ? 1 : 0,
                       );
-                # Trim down any times >= 10 to avoid ugly Text::Simple line wrapping
-                my $elapsed = substr(sprintf("%f", $stat->{elapsed}), 0, 8) . "s";
-                $t->row( ( q{ } x $r[0] ) . $r[1],
-                     defined $r[2] ? $elapsed : '??');
+                $t->row( ( q{ } x $r[0] ) . $r[1], 
+                     defined $r[2] ? sprintf("%fs", $r[2]) : '??');
                 push(@results, \@r);
                 }
             );
@@ -123,51 +122,14 @@ sub _get_uid {
 
     my $visitor = Tree::Simple::Visitor::FindByUID->new;
     $visitor->searchForUID($uid);
-    $self->accept($visitor);
+    $self->{tree}->accept($visitor);
     return $visitor->getResult;
-}
-
-sub addChild {
-    my $self = shift;
-    my $node = $_[ 0 ];
-
-    my $stat = $node->getNodeValue;
-
-    # do we need to fake $stat->{ t } ?
-    if( $stat->{ elapsed } ) {
-        # remove the "s" from elapsed time
-        $stat->{ elapsed } =~ s{s$}{};
-    }
-
-    $self->tree->addChild( @_ );
-}
-
-sub setNodeValue {
-    my $self = shift;
-    my $stat = $_[ 0 ];
-
-    # do we need to fake $stat->{ t } ?
-    if( $stat->{ elapsed } ) {
-        # remove the "s" from elapsed time
-        $stat->{ elapsed } =~ s{s$}{};
-    }
-
-    $self->tree->setNodeValue( @_ );
-}
-
-sub getNodeValue {
-    my $self = shift;
-    $self->tree->getNodeValue( @_ )->{ t };
-}
-
-__PACKAGE__->meta->make_immutable();
+} 
 
 1;
 
 __END__
 
-=for stopwords addChild getNodeValue mysub rollup setNodeValue
-
 =head1 NAME
 
 Catalyst::Stats - Catalyst Timing Statistics Class
@@ -212,7 +174,7 @@ be like this:
     $c->stats->profile("completed second part of critical bit");
     # more code
     ...
-    $c->stats->profile(end => "mysub");
+    $c->stats->profile(end => "mysub"); 
   }
 
 Supposing mysub was called from the action "process" inside a Catalyst
@@ -239,7 +201,7 @@ part 0.111s.
 
 =head2 new
 
-Constructor.
+Constructor. 
 
     $stats = Catalyst::Stats->new;
 
@@ -258,7 +220,7 @@ Enable or disable stats collection.  By default, stats are enabled after object
 
 Marks a profiling point.  These can appear in pairs, to time the block of code
 between the begin/end pairs, or by themselves, in which case the time of
-execution to the previous profiling point will be reported.
+execution to the previous profiling point will be reported.  
 
 The argument may be either a single comment string or a list of name-value
 pairs.  Thus the following are equivalent:
@@ -303,13 +265,6 @@ The profiling point will be ignored if the UID has not been previously defined.
 Returns the UID of the current point in the profile tree.  The UID is
 automatically assigned if not explicitly given.
 
-=head2 created
-
-    ($seconds, $microseconds) = $stats->created;
-
-Returns the time the object was created, in C<gettimeofday> format, with
-Unix epoch seconds followed by microseconds.
-
 =head2 elapsed
 
     $elapsed = $stats->elapsed
@@ -338,36 +293,20 @@ from the previous profiling point.
 The 'rollup' flag indicates whether the reported time is the rolled up time for
 the block, or the elapsed time from the previous profiling point.
 
-=head1 COMPATIBILITY METHODS
-
-Some components might expect the stats object to be a regular Tree::Simple object.
-We've added some compatibility methods to handle this scenario:
-
-=head2 accept
-
-=head2 addChild
-
-=head2 setNodeValue
-
-=head2 getNodeValue
-
-=head2 traverse
 
 =head1 SEE ALSO
 
-L<Catalyst>
+L<Catalyst>.
 
-=head1 AUTHORS
+=head1 AUTHOR
 
-Catalyst Contributors, see Catalyst.pm
+Jon Schutz
 
 =head1 COPYRIGHT
 
-This library is free software. You can redistribute it and/or modify
+This program is free software, you can redistribute it and/or modify
 it under the same terms as Perl itself.
 
 =cut
 
-__PACKAGE__->meta->make_immutable;
-
 1;
