@@ -17,7 +17,7 @@ EOF
 sub body : Local {
     my ( $self, $c ) = @_;
     
-    my $file = "$FindBin::Bin/../lib/TestApp/Controller/Action/Streaming.pm";
+    my $file = "$FindBin::Bin/01use.t";
     my $fh = IO::File->new( $file, 'r' );
     if ( defined $fh ) {
         $c->res->body( $fh );
